#include "GMAPI.hpp"
#include <Windows.h>
#include "gmlua.h"
#define EXPORT extern "C" __declspec(dllexport)

EXPORT double gmluaInit(double get_function_address)
{
    if (gm::init(get_function_address))
    {
        return true;
    }
    else return false;
}
EXPORT double gmluaFree()
{
    gm::shutdown();
    return 1;
}

EXPORT double setErrorScript(double sid)
{
	//gm::show_message("set errorScript");
	errorScript = (int)sid;
	return true;
}

EXPORT double newState()
{
	//gm::show_message("newState");
	luaFile* f = new luaFile();
	f->lua = lua_open();
	if (!f->lua)
	{
		delete f;
		return 0;
	}
	lua_atpanic(f->lua, _gmPanic);
	luaL_openlibs(f->lua);

	return (double)(size_t)f;
}

EXPORT double destroyState(double i)
{
	//gm::show_message("destroyState");
	luaFile* f = (luaFile*)(size_t)i;
	if (f == NULL) return false;
	lua_close(f->lua);
	delete f;
	return true;
}

EXPORT double loadFile(double i, char* filename)
{
	//gm::show_message("loadFile");
	luaFile* f = (luaFile*)(size_t)i;
	if (luaL_loadfile(f->lua, filename) == 0)
	{
		if (lua_pcall(f->lua, 0, LUA_MULTRET, 0) == 0) return true;
	}

	// Something bad happened
	_gmError(f->lua);
	return false;
}

EXPORT double loadString(double i, char* str)
{
	//gm::show_message("loadString");
	luaFile* f = (luaFile*)(size_t)i;
	if (f == NULL) return false;

	if (luaL_loadstring(f->lua, str) == 0)
	{
		if(lua_pcall(f->lua, 0, LUA_MULTRET, 0) == 0) return true;
	}

	// Something bad happened
	_gmError(f->lua);
	return false;
}

EXPORT double resetIndex(double i)
{
	//gm::show_message("resetIndex");
	luaFile* f = (luaFile*)(size_t)i;
	f->resetIndex();
	return true;
}

EXPORT double setIndex(double i, char* name)
{
	//gm::show_message("setIndex");
	luaFile* f = (luaFile*)(size_t)i;
	f->setIndex(name);
	return true;
}

EXPORT double registerScript(double i, char* name, double callback)
{
	//gm::show_message("registerScript");
	luaFile* f = (luaFile*)(size_t)i;
	f->addScript(name, (int)callback);
	return true;
}

EXPORT double callFunction(double i, char *name, double list)
{
	//gm::show_message("callFunction");
	luaFile* f = (luaFile*)(size_t)i;
	f->call(name, (int)list);
	return true;
}

EXPORT double callEvent(double i, char *name, double list)
{
	//gm::show_message("callEvent");
	luaFile* f = (luaFile*)(size_t)i;

	f->call(name, (int)list);
	gm::Value val = gm::ds_list_find_value((int)list, 0);
	if (val.getReal() == 1)
		return false;
	else
		return true;
}

EXPORT double addInternal(double i)
{
	luaFile* f = (luaFile*)(size_t)i;

	lua_pushcclosure(f->lua, _getGMFunction, 0);
	lua_setglobal(f->lua, "getGMFunction");

	lua_pushcclosure(f->lua, _callGMFunction, 0);
	lua_setglobal(f->lua, "callGMFunction");

	return true;
}

int _getGMFunction(lua_State* L)
{
	//gm::show_message("_getGMFunction");
	int argCount = lua_gettop(L);

	if (argCount != 1)
	{
		lua_pushstring(L, "getGMFunction takes only one parameter.");
		lua_error(L);
		return 0;
	}

	if (!lua_isstring(L, 1))
	{
		lua_pushstring(L, "getGMFunction: first parameter must be a string");
		lua_error(L);
		return 0;
	}

	gm::String str = lua_tostring(L, 1);
	gm::Real real = gm::get_function_address(str);
	gm::show_message(str);
	lua_pushnumber(L, (int)real.real);
	return 0;
}

int _callGMFunction(lua_State* L)
{
	gm::show_message("_callGMFunction");
	int argCount = lua_gettop(L);
	if (argCount < 1)
	{
		lua_pushstring(L, "callGMFunction requires at least one parameter.");
		lua_error(L);
		return 0;
	}

	if (!lua_isnumber(L, 1))
	{
		lua_pushstring(L, "callGMFunction: provided function isn't a number.");
		lua_error(L);
		return 0;
	}

	int func = (int)lua_tonumber(L, 1);
	if (func < 0)
	{
		lua_pushstring(L, "callGMFunction: provided function is bad.");
		lua_error(L);
		return 0;
	}

	if(func >= 0) {
		gm::Function function = gm::functions[func];
		gm::Value ret;
		argCount = argCount - 2;
		if(argCount > 0) {
			gm::Value* args = new gm::Value[argCount + 1];
			args[argCount] = func;
			for (int i = 0; i < argCount - 1; i++)
			{
				if (lua_isstring(L, i + 2))
					args[i].setStr(lua_tostring(L, i + 2));
				else if (lua_isnumber(L, i + 2))
					args[i].setReal(lua_tonumber(L, i + 2));
				else
				{
					lua_pushstring(L, "callGMFunction: parameter(s) must be a string or number (not integer)");
					lua_error(L);
					delete [] args;
					return 0;
				}
			}

			// Handle the arguments
			ret = gm::docall(function, argCount, args);

			// Release the arguments
			delete [] args;
		}
		else ret = gm::call(function);

		// Handle the return value
		if (ret.type == gm::Value::REAL)
		{
			lua_pushnumber(L, ret.real);
		} else if (ret.type == gm::Value::STRING) {
			lua_pushstring(L, ret.str.getData());
		} else {
			lua_pushstring(L, "callGMFunction: return value isn't a real ot string, something is wrong.");
			lua_error(L);
			return 0;
		}

		return 1;
	}
	return 0;
}
